@@ -62,7 +62,7 @@ pub struct RuleRef
 pub struct AssemblyOutput
 {
 	pub binary: util::BitVec,
-	pub symbols: asm::SymbolManager,
+	pub state: State,
 }
 
 
@@ -87,7 +87,7 @@ impl Assembler
 	
 	
 	pub fn assemble(
-        &mut self,
+        mut self,
         report: diagn::RcReport,
 		fileserver: &dyn util::FileServer,
 		max_iterations: usize)
@@ -172,8 +172,8 @@ impl Assembler
 
 				return Ok(AssemblyOutput
 				{
+					state: self.state,
 					binary: full_output,
-					symbols: std::mem::replace(&mut self.state.symbols, asm::SymbolManager::new()),
 				});
 			}
 
@@ -705,6 +705,32 @@ impl State
 			}
 		}
 
+		// Retain only the candidates which produced the fewest bits
+		let mut smallest_output = usize::MAX;
+		for c in &successful_candidates
+		{
+			if let expr::Value::Integer(ref bigint) = c.1
+			{
+				if let Some(size) = bigint.size
+				{
+					smallest_output = std::cmp::min(smallest_output, size);
+				}
+			}
+		}
+
+		successful_candidates.retain(|c|
+		{
+			if let expr::Value::Integer(ref bigint) = c.1
+			{
+				if let Some(size) = bigint.size
+				{
+					return size == smallest_output;
+				}
+			}
+
+			false
+		});
+		
 		if successful_candidates.len() > 0
 		{
 			if final_pass
@@ -712,7 +738,7 @@ impl State
 				if successful_candidates.len() > 1
 				{
 					let _guard = report.push_parent(
-						"multiple matches for instruction",
+						"multiple matches for instruction with the same output size",
 						&invocation.span);
 
 					for c in successful_candidates
